@@ -334,7 +334,6 @@ class ConvNet(pl.LightningModule):
     def training_step(self, train_batch, batch_idx):
         x_batch, y_batch = train_batch
         x_batch = self.reshape_x(x_batch)
-            
         y_batch = self.sample_weight * y_batch
 
         Yhat = self.forward(x_batch)
@@ -440,6 +439,198 @@ class ConvNet(pl.LightningModule):
 
         return optimizer
 
+
+class SegNet(pl.LightningModule):
+    r"""SegNet. Use this when using CNNs to perform Image Segmentation. 
+
+    Note:
+
+        SegNet is the parent class for CNNs(VGG). It is also compatible with other CNNs
+        that are written in pytorch_lightning. 
+
+    Attributes:
+
+        model_type (LightningModule): The class of the model that you will use.
+
+        n_classes (int, optional): Number of classes in the classification task. Leave it as None in regression tasks.
+
+        sample_weight (torch.Tensor or np.ndarray): Sample weights.
+
+        config (dict) : Hyperparameters that can be shared among different models. e.g., optimizer_type, lr, weight_decay, batch_size, bn, dropout_p, etc.
+"""
+    def __init__(
+        self,
+        n_classes : Optional[int] = None,
+        sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
+        dataformats : Optional[str] = "NCHW",
+        **config : Any
+    )->None:
+        r"""
+        Args: 
+
+            n_classes (int, optional): Number of classes in the classification task. Leave it as None in regression tasks.
+                Default: ``None``
+            
+            sample_weight (torch.Tensor or np.ndarray): Sample weights.
+                Default: ``None``
+
+            **config (Any) : Hyperparameters that can be shared among different models. e.g., optimizer_type, lr, weight_decay, batch_size, bn, dropout_p, etc.
+        """
+        super(ConvNet, self).__init__()
+        
+        self.n_classes = n_classes
+        self.sample_weight = sample_weight
+        self.dataformats = dataformats
+        self.save_hyperparameters(*config)
+        
+
+    @property
+    def criterion(self):   
+        r"""Criterion for loss function.
+
+        Returns:
+
+            See astrape.utilities.utils for details.
+        """
+        if self.n_classes is None:
+            raise AssertionError(f"Please pass the # of classes.")
+        else:
+            criterion = SegCELossMetrics(self.n_classes)
+        return criterion
+
+    def reshape_x(self, x_batch):
+        dims = x_batch.shape[1:]
+        
+        if len(dims) == 1:
+            raise ValueError(f"Wrong dimension {dims} for the input.")
+        elif len(dims) == 2:
+            height = dims[0]
+            width = dims[1]
+            x_batch = x_batch.reshape(-1, 1, int(height), int(width))
+        elif len(dims) == 3:
+            dims = rearrange_dims(dims=dims, in_format=self.dataformats, dataformats="NCHW")
+            x_batch = x_batch.reshape(-1, *dims)                
+        else:
+            raise ValueError(f"Wrong dimension {dims} for the input.")
+
+        return x_batch
+
+    def on_before_batch_transfer(self, batch: Any, dataloader_idx=0):
+        batch[0] = self.reshape_x(batch[0])
+        return batch
+    
+    def on_after_batch_transfer(self, batch, dataloader_idx=0):
+        # modify this part if you want to perform model reweigthing.
+        self.sample_weight = torch.ones_like(batch[1]).type_as(batch[1])
+        return batch
+    
+    def training_step(self, train_batch, batch_idx):
+        x_batch, y_batch = train_batch
+        x_batch = self.reshape_x(x_batch)
+        y_batch = self.sample_weight * y_batch
+
+        Yhat = self.forward(x_batch)
+        if self.n_classes is None:
+            loss = self.criterion(Yhat.squeeze(), y_batch)
+            self.log('train/loss', loss)
+            return {'loss' : loss}
+        else:    
+            if self.n_classes > 2:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.LongTensor))
+            else:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.FloatTensor))
+            self.log('train/loss', loss)
+            self.log('train/acc', acc)
+            self.log('train/auc', auc)
+            log_dict = {'train/loss' : loss, 'train/acc' : acc, 'train/auc' : auc}
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc, "progress_bar" : log_dict}
+
+
+        
+    def validation_step(self, val_batch, batch_idx):
+        x_batch, y_batch = val_batch
+        x_batch = self.reshape_x(x_batch)
+        Yhat = self.forward(x_batch)
+        if self.n_classes is None:
+            loss = self.criterion(Yhat.squeeze(), y_batch)
+            self.log('val/loss', loss)
+            return {'loss' : loss}
+        else:    
+            if self.n_classes > 2:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.LongTensor))
+            else:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.FloatTensor))
+
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc}
+
+    def validation_epoch_end(self, outputs):
+        loss = torch.stack([x["loss"] for x in outputs]).mean()
+        acc = torch.stack([x["acc"] for x in outputs]).mean()
+        auc = torch.stack([x["auc"] for x in outputs]).mean()
+        log_dict = {'val/loss' : loss, 'val/acc' : acc, 'val/auc' : auc}
+        return {"log": log_dict, "progress_bar" : log_dict}
+
+    def predict_step(self, pred_batch, batch_idx, dataloader_idx=0):
+        x_batch, y_batch = pred_batch
+        x_batch = self.reshape_x(x_batch)
+        Yhat = self.forward(x_batch)
+        if self.n_classes is None:
+            loss = self.criterion(Yhat.squeeze(), y_batch)
+            self.log('pred/loss', loss)
+            return {'loss' : loss}
+        else:    
+            if self.n_classes > 2:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.LongTensor))
+            else:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.FloatTensor))
+            self.log('pred/loss', loss)
+            self.log('pred/acc', acc)
+            self.log('pred/auc', auc)
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc}
+
+    def test_step(self, test_batch, batch_idx):
+        x_batch, y_batch = test_batch
+        x_batch = self.reshape_x(x_batch)
+        Yhat = self.forward(x_batch)
+        if self.n_classes is None:
+            loss = self.criterion(Yhat.squeeze(), y_batch)
+            self.log('test/loss', loss)
+            return {'loss' : loss}
+        else:    
+            if self.n_classes > 2:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.LongTensor))
+            else:
+                loss, acc, auc = self.criterion(Yhat.squeeze(), y_batch.type(torch.FloatTensor))
+            self.log('test/loss', loss)
+            self.log('test/acc', acc)
+            self.log('test/auc', auc)
+            return {'loss' : loss, 'acc' : acc, 'auc' : auc}
+
+    def reset_weights(self, module):
+        if isinstance(module, nn.Linear):
+            module.reset_parameters()
+            
+    def _init_weights(self, module) -> None:
+        if isinstance(module, nn.Linear):
+            nn.init.kaiming_normal_(module.weight)
+        elif isinstance(module, nn.BatchNorm1d):
+            nn.init.constant_(module.weight, 1)
+            nn.init.constant_(module.bias, 0)
+        elif isinstance(module, nn.Conv2d):
+            nn.init.kaiming_normal_(module.weight, mode="fan_out", nonlinearity="relu")
+            if module.bias is not None:
+                nn.init.constant_(module.bias, 0)
+
+    def configure_optimizers(self):
+        if self.hparams.optimizer_type == "adam":
+            optimizer = torch.optim.Adam(self.parameters(),
+                                     lr=self.hparams.lr,
+                                     weight_decay=self.hparams.weight_decay)
+        elif self.hparams.optimizer_type == "sgd":
+            optimizer = torch.optim.SGD(self.parameters(),
+                                        lr=self.hparams.lr)
+
+        return optimizer
 """
 class GenerativeNet(pl.LightningModule):
     def __init__(
